﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WpfApp2_tictactoe.BL
{
    class TicTacLogic
    {
        const int GAMESIZE = 5;
        const int MUSTWIN = 3;

        char nextPlayer = 'O';

        public char[,] Board { get; private set; } // X; Y
        public int Width { get => Board.GetLength(0); }
        public int Height { get => Board.GetLength(1); }
        public event EventHandler GameOver;

        // Singleton
        public static TicTacLogic Single { get; private set; } = new TicTacLogic();
        private TicTacLogic()
        {
            Board = new char[GAMESIZE, GAMESIZE];
        }

        public char PlaceNext(int x, int y)
        {
            if (Board[x, y] == '\0')
            {
                Board[x, y] = nextPlayer;
                if (IsGameOver(x,y)) GameOver?.Invoke(this, EventArgs.Empty);
                nextPlayer = nextPlayer == 'O' ? 'X' : 'O';
            }
            return Board[x, y];
        }
        private bool IsGameOver(int x, int y)
        {
            int horiz = 1 + CalcSame(x, y, 1, 0) + CalcSame(x, y, -1, 0);
            int vert = 1 + CalcSame(x, y, 0, 1) + CalcSame(x, y, 0, -1);
            int diag1 = 1 + CalcSame(x, y, 1, 1) + CalcSame(x, y, -1, -1);
            int diag2 = 1 + CalcSame(x, y, 1, -1) + CalcSame(x, y, -1, 1);
            return (horiz >= MUSTWIN || vert >= MUSTWIN || diag1 >= MUSTWIN || diag2 >= MUSTWIN);
        }
        private int CalcSame(int x, int y, int dx, int dy)
        {
            int cnt = -1;
            char c = Board[x, y];
            do
            {
                cnt++;
                x += dx;
                y += dy;
            } while (x >= 0 && y >= 0 && x < Width && y < Height && Board[x, y] == c);
            return cnt;
        }


    }
}
